
//
//cpu_load.c
//

/*
how to measure cpu load
- count time in os idle loop ...

every 10ms os switches tasks
if a task issues a break #0, it gives up the rest of the slot?
some tasks (well usb task) spin for their entire timeslice ...
most just call get_ioctl(), which includes a break #0
no idea abount task main()

-- could measure idle calls per second
this wonn't be a load percentage; but will give a measure
a low count => a high load

*/

typedef unsigned char  UCHAR;
typedef unsigned short USHORT;
typedef unsigned long  ULONG;
#include <tap.h>

#define ID_USERTAP 0xbdb01701
TAP_ID( ID_USERTAP );

TAP_PROGRAM_NAME("cpu_load v0.01");
TAP_AUTHOR_NAME("bdb");
TAP_DESCRIPTION("displays cpu load");
TAP_ETCINFO(__DATE__);

#define ptrfix(p) (p + _tap_startAddr)
#define pSysID          0xa3fffffa
extern int _appl_version;

dword fw_tap_table;
dword fw_tap_table_1288 = 0x80176f60;    //456:0x1288
dword fw_tap_table_1326 = 0x80179024;    //456:0x1326
//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
#include "TSRCommander.inc"

extern void TSRCommanderConfigDialog();
extern bool TSRCommanderExitTAP();
extern int is_tsr_commander_running(void);

//----------------------------------------------------------------------------
//
//tap load structures
//
typedef struct
{
    byte signature[32];					//TFAMIPS
    dword id;
    byte padding1[28];
    const char name[MAX_PROGRAM_NAME];
    const char authorName[MAX_AUTHOR_NAME];
    const char description[MAX_DESCRIPTION];
    const char etcStr[MAX_ETCINFO_STR];
} type_TAPHeader;

typedef struct
{
    // From libtap.a                                               12.88
    dword _initData;                                      //  00   827b37c4
    void *TAP_Main;                                       //  04   827a8b68
    dword _tap_gotStart;                                  //  08   00020be0
    dword _tap_gotEnd;                                    //  0c   000210f4
    dword _tap_textEnd;                                   //  10   0000c970
    dword _tap_fixupStart;                                //  14   000210ec
    dword _tap_fixupEnd;                                  //  18   000210f0
    void *TAP_EventHandler;                               //  1c   827a8d30
    dword _gp;                                            //  20   00020be0
    dword _end;                                           //  24   000246c0
                                                          //
    void *pTAP_SystemProc;                                //  28   80177a54
    void *_appl_version;                                  //  2c   00001288
    //etc
} type_TAPEntryPoints;

typedef struct
{
    dword firmwaregp;
    dword tapgp;
    dword unknown1;                         //tap malloc base address, before + 7ffe, & 0xFFFF8000
    type_TAPHeader *header;                 //after & 0xffff8000
    type_TAPEntryPoints *entryPoints;
    dword loadAddress;
    dword *currentDir;                      //pointer to this taps current directory ?
    dword zero;                             //exit flag
} type_TAPProcess;

//----------------------------------------------------------------------------
//
//menu stuff
//
typedef struct {
    int value;
    char string[0x20];
} type_state;

typedef struct {
    int  id;
    char str[0x20];
}type_item;

typedef struct {
    int  id;
    int  visible;
    TYPE_Window win;
    int  rgn;
    char *fixme_titlep;
    type_item item[0x20];
}type_menu;

extern type_menu main_menu;
void ShowMenu(type_menu *menu);
void DeleteMenu(type_menu *menu);

//----------------------------------------------------------------------------
//
//main
//
int TAP_Main(void);
dword TAP_EventHandler(word event, dword param1, dword param2);
int main_menu_handler(type_menu *menu, dword param1);
int key_handler(dword param1);

int close(void);
void display_load(int min, int max, int count);
int rgn;							    // stores rgn-handle for osd
int load_enabled = 1;

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
bool TSRCommanderExitTAP()
{
    close();
    TAP_Exit();
    return FALSE;
}

void TSRCommanderConfigDialog()
{
    TAP_ExitNormal();
    ShowMenu(&main_menu);
}

int is_tsr_commander_running(void)
{
    int i;
    type_TAPProcess *TAPProcess;

    for (i=0; i<16; i++) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (i*32));
        if (TAPProcess->firmwaregp) {
            if (TAPProcess->header) {
                if (TAPProcess->header->id == 0x807a10b2) {  //tsr_commander
                    return 1;
                }
                if (TAPProcess->header->id == 0x81001021) {  //tap_commander
                    return 1;
                }
            }
        }
    }
    return 0;
}

//----------------------------------------------------------------------------
//
//main
//
char *tap_name;
type_menu main_menu = {
    0,
    0,
    {0},
    0,
    (char *)__tap_program_name__,
    {
        {3, "xxx",              },

        {0, "enable load",      },
        {1, "disable load",     },
        {2, "Exit Tap",         },
    },
};

int TAP_Main(void)
{
    word fw_sysid = *(word*)pSysID;
    int ret;

    tap_name =  (char *)__tap_program_name__;
    TAP_Print("Opening %s\n", tap_name);

    //determine support for different firmwares
    if ((fw_sysid==456)&&(_appl_version==0x1288)) {
        fw_tap_table = fw_tap_table_1288;
    }
    else if ((fw_sysid==456)&&(_appl_version==0x1326)) {
        fw_tap_table = fw_tap_table_1326;
    }
    else {
        TAP_Print("Unsupported platform - %d:%04x\n", *(word*)pSysID, _appl_version);
        return 0;
    }

    main_menu.visible = 0;
    TSRCommanderInit(0, TRUE);
    return 1;                 //0=application mode, 1=tsr mode
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
    ULONG state, subState;
    int now;
    static int next = 0;
    static int idle = 0;
    static int min = 10;
    static int max = 5700;

    switch (event) {
        case EVT_IDLE:
            //count idles in last 1000ms
            now = TAP_GetTick();
            if (next == 0) next = now + 100;
            idle ++;
            if (now > next) {
                //log idle
                if (idle < min) min = idle;
                if (idle > max) max = idle;
//                TAP_Print("min=%d, max=%d, idle=%d\n", min, max, idle);
                if (load_enabled) display_load(min, max, idle);
                idle = 0;
                next = now + 100;
            }

            TSRCommanderWork();
            break;

        case EVT_KEY:
            TAP_GetState(&state, &subState);
            if (main_menu.visible) { // handling keys when options window is shown
                param1 = main_menu_handler(&main_menu, param1);
            }
            else
            if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                param1 = key_handler(param1);
            }
            break;
        default:
            break;
    }
    return param1;
}

int main_menu_handler(type_menu *menu, dword param1)
{
    int selection;

    switch (param1) {
        case RKEY_Ok:
            selection = TAP_Win_GetSelection(&menu->win);
            DeleteMenu(menu);
            TAP_EnterNormal();
            switch (selection) {
                case 0:
                    //enable display
                    load_enabled = 1;
                    rgn = TAP_Osd_Create(0, 0, 720, 576, 0, FALSE);             // create rgn-handle
                    return 0;
                case 1:
                    //disable display
                	TAP_Osd_Delete(rgn);                                        // release rgn-handle
                    load_enabled = 0;
                    return 0;
                case 2:
                    close();
                    TAP_Exit();
                    return 0;
            }
            break;
        case RKEY_Recall:
            DeleteMenu(menu);
            TAP_EnterNormal();
            return param1; // cascades key to other TAP's
        case RKEY_Exit:
            DeleteMenu(menu);
            TAP_EnterNormal();
            return 0; // exit without Cascading
        default:
            TAP_Win_Action(&menu->win, param1); // send all other key's to menu-Win
            return 0;
    }
    return param1;
}

int key_handler(dword param1)
{
    switch(param1) {
        //only if tsrcommander is not running
        case RKEY_Recall:
            if (!is_tsr_commander_running()) {
                TAP_ExitNormal();
                ShowMenu(&main_menu);
                return 0;
            }
            break;
    }
    return param1;
}


int close(void)
{
    DeleteMenu(&main_menu);

    TSRCommanderErase();
    TAP_Print("Closing %s\n", tap_name);
    return 0;
}

//----------------------------------------------------------------------------
//
//menu stuff
//

void ShowMenu(type_menu *menu)
{
    int i;
    int num_items;
    int y,h;

	if (!menu->visible) {
	    menu->rgn = TAP_Osd_Create(0, 0, 720, 576, 0, FALSE);
	    TAP_Win_SetDefaultColor(&menu->win);
	    num_items = menu->item[0].id;
	    h = num_items * 20;
	    h = (h < 80) ? 80 : h;
	    h = (h > 280) ? 280 : h;
	    y = 248 - (num_items * 10);
	    y = (y < 50) ? 50 : y;
	    y = (y > 250) ? 250 : y;
	    //x,y,width,height
	    TAP_Win_Create(&menu->win, menu->rgn, 200, y, 300, h, FALSE, FALSE);
        TAP_Win_SetTitle(&menu->win, ptrfix(menu->fixme_titlep), 0, FNT_Size_1622);
//        TAP_Win_SetTitle(&menu->win, menu->title, 0, FNT_Size_1622);
	    for (i=1; i<=menu->item[0].id; i++) {
    	    TAP_Win_AddItem(&menu->win, menu->item[i].str);
	    }
	    TAP_Win_SetSelection(&menu->win, 0);
	    menu->visible = 1;
	}
}

void DeleteMenu(type_menu *menu)
{
	if (menu->visible) {
	    TAP_Win_Delete(&menu->win);
    	TAP_Osd_Delete(menu->rgn);
	    menu->visible = 0;
    }
}

//----------------------------------------------------------------------------
//yellow on purple background

void display_load(int min, int max, int count)
{
    char strTemp[0x40];

//    sprintf(strTemp, " %d  %d  %d ", min, count, max);
    sprintf(strTemp, " cpu load = %d ", ((max-count-min)*100)/(max-min));
	TAP_Osd_PutS(rgn, 544-25, 36, (720)-40, strTemp,            // show message
		RGB(31,31,0), RGB(20,0,20), 0, FNT_Size_1622, FALSE, ALIGN_LEFT);
}

//----------------------------------------------------------------------------

